/* Copyright 2016 Dirk Duschinger */

//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include "CursesOutputHandler.h"

CursesOutputHandler::~CursesOutputHandler() {
}

CursesOutputHandler::CursesOutputHandler(ROOTReader* rr)
  : m_iWidthLeftPanel(40)
  , m_iPanelOffset(0)
  , m_iPanelCurserPosition(0)
  , m_RR(rr) {
  initialize();
}

void CursesOutputHandler::initialize() {
  initscr();
  noecho();
  cbreak();
  curs_set(0);
  if (has_colors()) {
    start_color();
    initcursescolors();
  }
  UpdateResolution();
  DrawFrame();

  m_wHelp = newwin(m_iHelpHeight, m_iHelpWidth, m_iPadY, m_iHelpX);
  m_wSave = newwin(m_iSaveHeight, m_iSaveWidth, m_iSaveY, m_iSaveX);
  m_wPad = newwin(m_iPadHeight, m_iPadWidth, m_iPadY, m_iPadX);
  m_wPanel = newwin(m_iPanelHeight, m_iPanelWidth, m_iPanelY, m_iPanelX);
  m_wDEBUG = newwin(50, 50, 1, 20);

  m_pPanels[0] = new_panel(m_wPanel);
  m_pPanels[1] = new_panel(m_wPad);
  m_pPanels[2] = new_panel(m_wHelp);
  m_pPanels[3] = new_panel(m_wSave);
  m_pPanels[4] = new_panel(m_wDEBUG);

  m_pPanelsData[0].hide = FALSE;
  m_pPanelsData[1].hide = FALSE;
  m_pPanelsData[2].hide = TRUE;
  m_pPanelsData[3].hide = TRUE;
  m_pPanelsData[4].hide = TRUE;

  set_panel_userptr(m_pPanels[0], &m_pPanelsData[0]);
  set_panel_userptr(m_pPanels[1], &m_pPanelsData[1]);
  set_panel_userptr(m_pPanels[2], &m_pPanelsData[2]);
  set_panel_userptr(m_pPanels[3], &m_pPanelsData[3]);
  set_panel_userptr(m_pPanels[4], &m_pPanelsData[4]);

  hide_panel(m_pPanels[2]);
  hide_panel(m_pPanels[3]);
  hide_panel(m_pPanels[4]);
  UpdateScreen();
}

void CursesOutputHandler::reinitialize() {
  UpdateResolution();
  werase(stdscr);
  DrawFrame();
  mvwin(m_wHelp, m_iHelpY, m_iHelpX);
  wresize(m_wHelp, m_iHelpHeight, m_iHelpWidth);
  mvwin(m_wSave, m_iSaveY, m_iSaveX);
  wresize(m_wSave, m_iSaveHeight, m_iSaveWidth);
  mvwin(m_wPad, m_iPadY, m_iPadX);
  wresize(m_wPad, m_iPadHeight, m_iPadWidth);
  wresize(m_wPanel, m_iPanelHeight, m_iPanelWidth);
  UpdateScreen();
}


void CursesOutputHandler::initcursescolors() {
  for (int16_t f = 0; f != 8; f++) {
    for (int16_t b = 0; b != 8; b++) {
      if ((f == 0) && (b == 0))
        continue;
      init_pair(f+b*8, f, b);
    }
  }
}

void CursesOutputHandler::CloseHelp() {
  hide_panel(m_pPanels[2]);
}

void CursesOutputHandler::CloseSave() {
  mvwaddstr(m_wSave, 1, 11, "                        ");
  hide_panel(m_pPanels[3]);
}

int CursesOutputHandler::GetCursesXRes() {
  return m_iCursesXRes;
}

// int CursesOutputHandler::GetCursesYRes(){
//   return m_iCursesYRes;
// }

// WINDOW* CursesOutputHandler::GetDEBUGWindow(){
//   return m_wDEBUG;
// }

int CursesOutputHandler::GetOffsetLeftPanel() {
  return m_iPanelOffset;
}

ROOTPanelEntry* CursesOutputHandler::GetPanelEntry(int y) {
  int pos = y;
  return GetPanelEntry(m_RR->GetRPE(), &pos);
}

ROOTPanelEntry* CursesOutputHandler::GetPanelEntry(ROOTPanelEntry* rpe,
                                                   int* y) {
  mvwaddstr(m_wDEBUG, *y, 0, tostr(rpe).c_str());

  if (*y <= 0)
    return rpe;
  (*y)--;

  if (rpe->GetIsFolder()) {
    if (rpe->GetFold())
      return NULL;
  }
  for (size_t i = 0; i != rpe->GetChildren().size(); i++) {
    ROOTPanelEntry* tmp_RPE = GetPanelEntry(rpe->GetChildren().at(i), y);
    if ((*y == 0) && (tmp_RPE != NULL))
      return tmp_RPE;
  }
  return NULL;
}

int* CursesOutputHandler::GetPanelHeight() {
  return &m_iPanelHeight;
}


WINDOW* CursesOutputHandler::GetPadWindow() {
  return m_wPad;
}

WINDOW* CursesOutputHandler::GetPanelWindow() {
  return m_wPanel;
}

int CursesOutputHandler::GetWidthLeftPanel() {
  return m_iWidthLeftPanel;
}

void CursesOutputHandler::OpenHelp() {
  show_panel(m_pPanels[2]);
  UpdateScreen();
}

char* CursesOutputHandler::OpenSave() {
  show_panel(m_pPanels[3]);
  echo();
  char* c = new char();
  mvwgetstr(m_wSave, 1, 11, c);
  noecho();
  return c;
}

void CursesOutputHandler::SetOffsetLeftPanel(int OffsetLeftPanel) {
  m_iPanelOffset = OffsetLeftPanel;
}

void CursesOutputHandler::SetWidthLeftPanel(int WidthLeftPanel) {
  m_iWidthLeftPanel = WidthLeftPanel;
}

void CursesOutputHandler::UpdateScreen() {
  update_panels();
  doupdate();
}

void CursesOutputHandler::DrawFrame() {
  box(stdscr, 0, 0);
  mvwaddch(stdscr, 0, m_iWidthLeftPanel, ACS_TTEE);
  mvwvline(stdscr, 1, m_iWidthLeftPanel, ACS_VLINE, m_iCursesYRes-1);
  mvwaddch(stdscr, m_iCursesYRes, m_iWidthLeftPanel, ACS_BTEE);
}

void CursesOutputHandler::UpdateResolution() {
  getmaxyx(stdscr, m_iCursesYRes, m_iCursesXRes);
  m_iCursesYRes--;
  m_iCursesXRes--;

  m_iPanelX = 1;
  m_iPanelY = 1;
  m_iPanelHeight = m_iCursesYRes-1;
  m_iPanelWidth = m_iWidthLeftPanel-1;

  m_iPadX = m_iWidthLeftPanel+1;
  m_iPadY = 1;
  m_iPadHeight = m_iCursesYRes-1;
  m_iPadWidth = m_iCursesXRes - m_iWidthLeftPanel -1;

  m_iHelpX = m_iWidthLeftPanel+1;
  m_iHelpY = 1;
  m_iHelpHeight = 10;
  m_iHelpWidth = 60;
  m_iSaveX = m_iWidthLeftPanel+1;
  m_iSaveY = 1;
  m_iSaveHeight = 3;
  m_iSaveWidth = 44;
}

void CursesOutputHandler::UpdatePanelSelection(int CurserY) {
  if (m_iPanelCurserPosition != CurserY) {
    for (int i = 0; i < m_iPanelWidth; i++) {
      chtype character;
      wcolor_set(m_wPanel, ((mvwinch(m_wPanel,
                                     m_iPanelCurserPosition,
                                     i) & A_COLOR) >> 8)%8, 0);
      character = mvwinch(m_wPanel, m_iPanelCurserPosition, i) & A_CHARTEXT;
      mvwaddch(m_wPanel, m_iPanelCurserPosition, i, character);
    }
  }
  for (int i=0; i < m_iPanelWidth; i++) {
    chtype character;
    wcolor_set(m_wPanel, ((mvwinch(m_wPanel,
                                   CurserY, i) &
                           A_COLOR)>>8)%8+56, 0);
    character = mvwinch(m_wPanel, CurserY, i) & A_CHARTEXT;
    mvwaddch(m_wPanel, CurserY, i, character);
  }
  wcolor_set(m_wPanel, 0, 0);
  m_iPanelCurserPosition = CurserY;
}

void CursesOutputHandler::WritePanel() {
  int yOffset = m_iPanelOffset;
  int xOffset = 0;

  werase(m_wPanel);
  wcolor_set(m_wPanel, m_RR->GetRPE()->GetColor(), 0);
  mvwaddnstr(m_wPanel,
             yOffset,
             xOffset,
             m_RR->GetRPE()->GetName().c_str(),
             m_iPanelWidth);
  wcolor_set(m_wPanel, 0, 0);

  WritePanelRecursive(m_RR->GetRPE(), yOffset, xOffset);
}

void CursesOutputHandler::WritePanelRecursive(ROOTPanelEntry* rpe,
                                              int& yOffset,
                                              int xOffset) {
  xOffset++;
  if (rpe->GetFold())
    return;
  for (size_t i=0; i != rpe->GetChildren().size(); i++) {
    yOffset++;
    wcolor_set(m_wPanel,
               rpe->GetChildren().at(i)->GetColor(),
               0);
    mvwaddnstr(m_wPanel,
               yOffset,
               0+xOffset,
               rpe->GetChildren().at(i)->GetName().c_str(),
               m_iPanelWidth - xOffset);
    wcolor_set(m_wPanel, 0, 0);
    WritePanelRecursive(rpe->GetChildren().at(i), yOffset, xOffset);
  }
}

void CursesOutputHandler::WriteHelp() {
  box(m_wHelp, 0, 0);
  mvwaddch(m_wHelp, 2, 0, ACS_LTEE);
  mvwhline(m_wHelp, 2, 1, ACS_HLINE, m_iHelpWidth-2);
  mvwaddch(m_wHelp, 2, m_iHelpWidth-1, ACS_RTEE);

  mvwaddstr(m_wHelp, 1, 2, "Keyboard Commmands:");

  mvwaddstr(m_wHelp, 3, 2, "* arrow left/right : move separator");
  mvwaddstr(m_wHelp, 4, 2, "* arrow up/down    : navigate in panel");
  mvwaddstr(m_wHelp, 5, 2, "* page up/down     : scroll up/down by 10");
  mvwaddstr(m_wHelp, 6, 2,
            "* enter            : open/close files and folders;");
  mvwaddstr(m_wHelp, 7, 2,
            "                     draw from histogram, branch, canvas");
  mvwaddstr(m_wHelp, 8, 2, "* q                : quit");
}

void CursesOutputHandler::WriteSave() {
  mvwaddstr(m_wSave, 1, 2, "Save to:");
  box(m_wSave, 0, 0);
}
