/* Copyright 2016 Dirk Duschinger */

//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include "EpsLine.h"

EpsLine::EpsLine() {
  initialize(0, 0, 0, 0, 0);
}

EpsLine::EpsLine(double x1,
                 double y1,
                 double x2,
                 double y2,
                 int color) {
  this->initialize(x1, y1, x2, y2, color);
}

EpsLine::~EpsLine() {
}

void EpsLine::initialize(double x1,
                         double y1,
                         double x2,
                         double y2,
                         int color) {
  m_dX1 = x1;
  m_dY1 = y1;
  m_dX2 = x2;
  m_dY2 = y2;
  m_dDX = x2-x1;
  m_dDY = y2-y1;

  m_iColor = color;
  m_cSymbol = "-";
  double slope = atan2(m_dDY, m_dDX)/M_PI;

  if ((slope <  .125) && (slope > -1.25))
    m_cSymbol = "-";
  if ((slope <  .625) && (slope >  .375))
    m_cSymbol = "|";
  if ((slope < -.375) && (slope > -.625))
    m_cSymbol = "|";
  if ((slope <  .375) && (slope >  .125))
    m_cSymbol = "/";
  if ((slope < -.625) && (slope > -.875))
    m_cSymbol = "/";
  if ((slope <  .875) && (slope >  .625))
    m_cSymbol = "\\";
  if ((slope < -.125) && (slope > -.375))
    m_cSymbol = "\\";
}

void EpsLine::Draw(WINDOW* w, int eps_xres, int eps_yres) {
  int curses_xres, curses_yres;
  getmaxyx(w, curses_yres, curses_xres);
  double xscale = 1./(1.*eps_xres/(curses_xres-1));
  double yscale = 1./(1.*eps_yres/(curses_yres-1));

  double x1 = 1. * m_dX1 * xscale;
  double y1 = 1. * m_dY1 * yscale;
  double x2 = 1. * m_dX2 * xscale;
  double y2 = 1. * m_dY2 * yscale;

  // flip vertically
  y1 = curses_yres - y1 -1;
  y2 = curses_yres - y2 -1;

  double dx = x2 - x1;
  double dy = y2 - y1;
  double length = sqrt(dx*dx+dy*dy);
  double slope = atan2(dy, dx);
  for (int i=0; i < static_cast<int>(length)+1; i++) {
    int x = static_cast<int>(y1+i*sin(slope));
    int y = static_cast<int>(x1+i*cos(slope));
    int col = m_iColor;
    // int fg = col%8;
    int fg = static_cast<int>(col/8);
    int bg = (mvwinch(w, x, y) & A_COLOR) >> 11;
    wcolor_set(w, fg+bg*8, 0);
    mvwaddch(w, x, y, chtype(*(m_cSymbol)));
    wcolor_set(w, 0, 0);
  }
}
