/* Copyright 2016 Dirk Duschinger */

//////////////////////////////////////
// author : Dirk Duschinger         //
// email  : dirk.duschigner@cern.ch //
//////////////////////////////////////

#include <vector>

#include "ROOTReader.h"

ROOTReader::ROOTReader()
  : m_RPE(0)
  , m_iNFiles(0) {
}

ROOTReader::~ROOTReader() {
}

ROOTReader::ROOTReader(int argc, const char* files[])
  : m_RPE(0) {
  gErrorIgnoreLevel = 5000;
  m_iNFiles = argc;
  for (int i = 1; i < m_iNFiles; i++) {
    m_vFiles.push_back(new TFile(files[i]));
  }
}

void ROOTReader::ReadFiles() {
  delete m_RPE;
  m_RPE = new ROOTPanelEntry();
  m_RPE->SetParent(NULL);
  m_RPE->SetName("Loaded Files");
  m_RPE->SetFold(false);
  m_RPE->SetColor(3);
  std::vector<TFile*>::iterator it = m_vFiles.begin();
  for (; it != m_vFiles.end(); ++it) {
    ROOTPanelEntry* tmp_RPE = new ROOTPanelEntry();
    tmp_RPE->SetParent(m_RPE);
    tmp_RPE->SetName((*it)->GetName());
    tmp_RPE->SetFold(true);
    tmp_RPE->SetColor(3);
    m_RPE->AddChild(tmp_RPE);
    ReadFile(tmp_RPE, *it);
  }
}

void ROOTReader::ReadFile(ROOTPanelEntry* rpe, TDirectory* dir) {
  TObjLink* lnk = dir->GetListOfKeys()->FirstLink();
  while (lnk) {
    TObject* obj = dir->Get(lnk->GetObject()->GetName());
    ROOTPanelEntry* tmp_RPE = new ROOTPanelEntry();
    tmp_RPE->SetParent(rpe);
    tmp_RPE->SetName(obj->GetName());
    if (obj->InheritsFrom("TDirectory")) {
      tmp_RPE->SetFold(true);
      tmp_RPE->SetIsFolder(true);
      tmp_RPE->SetColor(2);
      ReadFile(tmp_RPE, dir->GetDirectory(lnk->GetObject()->GetName()));
      rpe->AddChild(tmp_RPE);
    } else if (obj->InheritsFrom("TTree")) {
      tmp_RPE->SetObject(obj);
      tmp_RPE->SetFold(true);
      tmp_RPE->SetIsFolder(true);
      tmp_RPE->SetColor(2);

      TTree* tree = dynamic_cast<TTree*>(obj);
      TObjArray* brancharray = tree->GetListOfBranches();
      for (Int_t i = 0; i != brancharray->GetEntriesFast(); i++) {
        ROOTPanelEntry* tmp_RPEchild = new ROOTPanelEntry();
        tmp_RPEchild->SetParent(tmp_RPE);
        tmp_RPEchild->SetName(brancharray->At(i)->GetName());
        tmp_RPEchild->SetColor(2);
        tmp_RPEchild->SetObject(brancharray->At(i));
        tmp_RPE->AddChild(tmp_RPEchild);
      }
      rpe->AddChild(tmp_RPE);
    } else {
      tmp_RPE->SetName(lnk->GetObject()->GetName());
      tmp_RPE->SetObject(obj);

      if (obj->InheritsFrom("TH1"))
        tmp_RPE->SetColor(4);
      else if (obj->InheritsFrom("TCanvas"))
        tmp_RPE->SetColor(5);
      else if (obj->InheritsFrom("TBranch"))
        tmp_RPE->SetColor(6);
      else
        tmp_RPE->SetColor(0);
      rpe->AddChild(tmp_RPE);
    }
    lnk = lnk->Next();
  }
}

ROOTPanelEntry* ROOTReader::GetRPE() {
  return m_RPE;
}
